
package dk.myrekrig;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.io.*;
import java.net.*;

class Data {
    // Data
    byte v1,v2,v3,rel;
    League[] leagues;
    Date[] dates;
    Author[] authors;
    Race[] races;

    Map/*<String,Race>*/ racemap;
    Map/*<String,League>*/ leaguemap;

    public Data(String url) throws IOException {
	StreamTokenizer st =
	    new StreamTokenizer
		(new BufferedReader
		    (new InputStreamReader
			(new URL(url)
			    .openConnection()
			    .getInputStream())));
	st.resetSyntax();
	st.whitespaceChars(0,31);
	st.wordChars(32,255);
	st.commentChar('#');
	
	// Read past the first separator
	while (!sepToken(st)) getToken(st);

	// Parse the release version
	v1 = Byte.parseByte(getToken(st));
	v2 = Byte.parseByte(getToken(st));
	v3 = Byte.parseByte(getToken(st));
	rel = Byte.parseByte(getToken(st));
	while (!sepToken(st)) getToken(st);

	// Parse the leagues
	leaguemap = new HashMap();
	List tl = new LinkedList();
	while (!sepToken(st)) {
	    League l = new League(getToken(st), getToken(st));
	    tl.add(l);
	    leaguemap.put(l.id, l);
	}
	leagues = (League[])tl.toArray(new League[tl.size()]);

	// Parse the result dates
	List dl = new LinkedList();
	while (!sepToken(st)) {
	    dl.add(new Date(getToken(st)));
	}
	dates = (Date[])dl.toArray(new Date[dl.size()]);

	// Parse the authors
	List al = new LinkedList();
	Map am = new HashMap();
	while (!sepToken(st)) {
	    Author a = new Author(getToken(st),getToken(st));
	    al.add(a);
	    am.put(a.id, a);
	}
	authors = (Author[])al.toArray(new Author[al.size()]);

	// Parse the races
	List rl = new LinkedList();
	racemap = new HashMap();
	while (!sepToken(st)) {
	    String id = getToken(st);
	    League[] leagues;
	    boolean active;
	    int id_klaf_index = id.indexOf('/');
	    if (id_klaf_index >= 0) {
		String leagues_chars = id.substring(id_klaf_index+1);
		id = id.substring(0, id_klaf_index);
		leagues = new League[leagues_chars.length()];
		for (int i = 0 ; i < leagues.length ; i++) {
		    leagues[i] = (League)leaguemap.get(leagues_chars.substring(i,i+1));
		}
		active = true;
	    } else {
		leagues = new League[0];
		active = false;
	    }
	    String title = getToken(st);
	    String author_name = getToken(st);
	    int anc = author_name.indexOf(',');
	    Author author, author2;
	    if (anc == -1) {
		author = (Author)am.get(author_name);
		author2 = null;
	    } else {
		author = (Author)am.get(author_name.substring(0,anc));
		author2 = (Author)am.get(author_name.substring(anc+1));
	    }
	    Date firstdate = new Date(getToken(st));
	    Date lastdate = new Date(getToken(st));
	    int mem_size = Integer.parseInt(getToken(st));
	    int source_size = Integer.parseInt(getToken(st));
	    String description = getToken(st);
	    Race r = new Race(id, title, author, author2, firstdate, lastdate, mem_size, source_size, description, active);
	    rl.add(r);
	    racemap.put(r.title, r);
	}
	races = (Race[])rl.toArray(new Race[rl.size()]);
    }
    
    static boolean sepToken(StreamTokenizer st) throws IOException {
	String s = getToken(st);
	if (s == null) return true;
	if (s.startsWith("---")) return true;
	st.pushBack();
	return false;
    }
    
    static String getToken(StreamTokenizer st) throws IOException {
	int type = st.nextToken();
	if (type == StreamTokenizer.TT_EOF) {
	    return null;
	}
	return st.sval;
    }

}
