
package dk.myrekrig;

import java.io.*;
import java.net.*;

class Game {
    Date date;
    int v1,v2,v3;
    long randomseed;
    Parameters min, max;
    Team[] teams;
    Battle[] battles;
    Team best_vict, best_perf, best_pres, best_comb, best_over;

    public Game(Data data, Date date, String url) throws IOException {
	this.date = date;
	BufferedReader br =
	    new BufferedReader
		(new InputStreamReader
		    (new URL(url)
			.openConnection()
			.getInputStream()));

	String line;

	// Version
	line = br.readLine();
	v1 = subInt(line,17,18);
	v2 = subInt(line,19,20);
	v3 = subInt(line,21,22);

	br.readLine();

	// NumAnts
	line = br.readLine();
	teams = new Team[subInt(line, 13, line.length())];

	br.readLine();
	br.readLine();
	br.readLine();
	br.readLine();

	// Parameters
	min = new Parameters();
	max = new Parameters();
	line = br.readLine(); min.mapwidth   = subInt(line, 16, 22); max.mapwidth   = subInt(line, 23, 29);
	line = br.readLine(); min.mapheight  = subInt(line, 16, 22); max.mapheight  = subInt(line, 23, 29);
	line = br.readLine(); min.startants  = subInt(line, 16, 22); max.startants  = subInt(line, 23, 29);
	line = br.readLine(); min.nfspace    = subInt(line, 16, 22); max.nfspace    = subInt(line, 23, 29);
	line = br.readLine(); min.nfmin      = subInt(line, 16, 22); max.nfmin      = subInt(line, 23, 29);
	line = br.readLine(); min.nfdiff     = subInt(line, 16, 22); max.nfdiff     = subInt(line, 23, 29);	
	br.readLine();
	br.readLine();
	br.readLine();
	br.readLine();
	line = br.readLine(); min.battlesize = subInt(line, 16, 22); max.battlesize = subInt(line, 23, 29);

	// NumBattles
	line = br.readLine();
	battles = new Battle[subInt(line, 16, 22)];

	// RandomSeed
	line = br.readLine();
	randomseed = subLong(line, 12, 22);

	br.readLine();
	br.readLine();

	// Teams
	for (int i = 0 ; i < teams.length ; i++) {
	    line = br.readLine();
	    teams[i] = new Team();
	    teams[i].letter = line.charAt(0);
	    teams[i].race = (Race)data.racemap.get(line.substring(2,12).trim());
	    teams[i].mem_size = Integer.parseInt(line.substring(12).trim());
	}

	br.readLine();
	br.readLine();

	// Battles
	for (int i = 0 ; i < battles.length ; i++) {
	    line = br.readLine();
	    battles[i] = new Battle();
	    battles[i].randomseed = subLong(line, 5, 15);
	    Parameters p = new Parameters();
	    p.mapwidth   = subInt(line, 15, 20);
	    p.mapheight  = subInt(line, 20, 25);
	    p.startants  = subInt(line, 25, 29);
	    p.nfspace    = subInt(line, 29, 33);
	    p.nfmin      = subInt(line, 33, 37);
	    p.nfdiff     = subInt(line, 37, 41);
	    p.battlesize = line.indexOf(' ', 42)-42;
	    battles[i].used = p;
	    Team[] t = new Team[p.battlesize];
	    for (int j = 0 ; j < t.length ; j++) {
		t[j] = teams[line.charAt(42+j)-'A'];
	    }
	    battles[i].teams = t;
	    int twidth = max.battlesize <= 5 ? 5 : max.battlesize;
	    battles[i].turns = subInt(line, 42+twidth, 48+twidth);
	    battles[i].term = line.charAt(49+twidth);
	    battles[i].winner = teams[line.charAt(51+twidth)-'A'];
	}
	
	br.readLine();
	br.readLine();

	// Results
	for (int i = 0 ; i < teams.length ; i++) {
	    line = br.readLine();
	    teams[i].battles = subInt(line, 10, 15);
	    teams[i].won     = subInt(line, 15, 20);
	    teams[i].bases   = subFloat(line, 20, 26);
	    teams[i].ants    = subInt(line, 26, 32);
	    teams[i].size    = subInt(line, 32, 38);
	    teams[i].ages    = subInt(line, 38, 44);
	    teams[i].comb    = subFloat(line, 44, 50)/100;
	    teams[i].time    = subFloat(line, 51, 58);
	    teams[i].vict    = subFloat(line, 58, 64)/100;
	    teams[i].perf    = subFloat(line, 65, 71)/100;
	    teams[i].pres    = subFloat(line, 72, 78)/100;
	    teams[i].over    = teams[i].vict == 0f ? 0f : teams[i].perf*teams[i].pres/teams[i].vict;

	    if (v1 < 2 || v2 < 4 || v3 < 5) {
		teams[i].time *= 1000;
	    }

	    if (i == 0 || teams[i].vict > best_vict.vict) best_vict = teams[i];
	    if (i == 0 || teams[i].perf > best_perf.perf) best_perf = teams[i];
	    if (i == 0 || teams[i].pres > best_pres.pres) best_pres = teams[i];
	    if (i == 0 || teams[i].comb > best_comb.comb) best_comb = teams[i];
	    if (i == 0 || teams[i].over > best_over.over) best_over = teams[i];
	}

	br.close();
    }

    private int subInt(String s, int a, int b) {
	return Integer.parseInt(s.substring(a,b).trim());
    }

    private long subLong(String s, int a, int b) {
	return Long.parseLong(s.substring(a,b).trim());
    }

    private float subFloat(String s, int a, int b) {
	return Float.parseFloat(s.substring(a,b).trim());
    }
				  
}
