import java.lang.reflect.*;

/**
 * This is an example implementation of the advice for Logging aspect.
 * Every advice must implement AdviceIF.
 */
public class Adv_Log implements AdviceIF {
  private boolean _ANY = false;

  /**
   * Constructor can specify whether the advice can be used
   * for any join point (for any public method of the shared object)
   * by setting the argument wildcard to true.
   */
  public Adv_Log(boolean wildcard) {
    _ANY = wildcard;
  }

  /**
   * Returns the name of the aspect of this advice.
   */
  public String getAspectName() {
    return "LOG";
  }

  /**
   * AspectRepository's beforeAdvice() call eventually invokes this method.
   */
  public int beforeAdvice(Object obj, Method m, Object[] args){
    if (_ANY) {
      System.out.println("LOG : beforeAdvice() " + m.getName());
    }
    return AspectRepository.RESUME;
  }

  /**
   * AspectRepository's afterAdvice() call eventually invokes this method.
   */
  public Object afterAdvice(Object obj, Method m, Object[] args, Object result){

    if (_ANY) {
      System.out.println("LOG : afterAdvice() " + m.getName());
    }
    return null;
  }
}
