import java.lang.Thread;
import java.lang.reflect.Method;

/**
 * This is an implementation of the advice for scheduling aspect
 * whose policy prefers GET.
 * Every advice must implement AdviceIF.
 * Every scheduling aspect must extend Preference to be able to access the task queues.
 */
public class Adv_PreferGet extends Preference implements AdviceIF {
  public String getAspectName() { return "SCHED"; }
  public String getPolicyName() { return "PreferGet"; }

  public int beforeAdvice(Object obj, Method m, Object[] args) {
    //System.out.println("    PGET scheduler.beforeAdvice" + Thread.currentThread().getName());
    return AspectRepository.RESUME;
  }

  /** Picks a task to be unblocked.
   *  Returns a reference to the task picked.
   */
  public Object afterAdvice(Object bb, Method m, Object[] args, Object result) {
    Task _picked = null;
    //System.out.println("    PGET scheduler.afterAdvice" + Thread.currentThread().getName());

      int item_cnt = ((BoundedBuffer) bb).getCnt();
      int buf_size = ((BoundedBuffer) bb).getSize();

      if (_prodQ.isEmpty() && _consQ.isEmpty()) {
        _picked = null;
        return _picked;
      }

      if (item_cnt == buf_size && _consQ.size() > 0) {
        // buffer is full and consumer is waiting
        _picked = _consQ.dequeue(); // pick a consumer.
        return _picked;
      }

      if (item_cnt == 0 && _prodQ.size() > 0) {
        // buffer empty and producer is waiting
        _picked = _prodQ.dequeue(); // pick a producer.
        return _picked;
      }

      if (_consQ.isEmpty()) {
        _picked = _prodQ.dequeue();
        return _picked;
      }
      if (_prodQ.isEmpty()) {
        _picked = _consQ.dequeue();
        return _picked;
      }

      // both consumers and producers are waiting
      // Prefer Get
      _picked = _consQ.dequeue(); // Prefer consumers.
      return _picked;
  }
}
