import java.lang.reflect.*;

/**
 * Every advice to be registered in the system must implement the AdviceIF interface.
 */

public interface AdviceIF {
  /**
   * getAspectName() returns the name of the aspect to which this advice belongs.
   */
  public String getAspectName();

  /**
   * beforeAdvice() is used by Pointcut's beforeAdvice() method.
   * obj is a reference to the shared object, for example, bounded buffer.
   * m is the shared object's method to be invoked if all the beforeAdvice() is successful.
   * args is the array that contains all the arguments passed to the method m.
   */
  public int beforeAdvice(Object obj, Method m, Object[] args);

  /**
   * afterAdvice() is used by Pointcut's afterAdvice() method.
   * obj is a reference to the shared object, for example, bounded buffer.
   * m is the shared object's method to be invoked if all the beforeAdvice() is successful.
   * args is the array that contains all the arguments passed to the method m.
   * result is the object returned by the method m of the shared object obj.
   */
  public Object afterAdvice(Object obj, Method m, Object[] args, Object result);
}
