import java.util.Observer;
import java.util.Random;
import java.util.Observable;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;


/**
 * Title:        Producer Consumer - Dynamic Weaver Framework
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

/**
 * Agent is an observer to the AspectRepository.
 * It is notified everytime a task is enqueued in one of the task queues
 * maintained in the scheduling advice.
 * It is notified of the number of items in the buffer, number of tasks in each queue,
 * then it decides the best scheduling policy to be used later on
 * based on knowleage learned from the training data and Bayesian algorithm.
 * Agent asks the AspectRepository to change the scheduling policy.
 */
public class AgentDialog extends JDialog implements Observer {
  JPanel panel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  JPanel jPanel3 = new JPanel();
  JButton btnHide = new JButton();
  JButton btnDisable = new JButton();
  JPanel jPanel1 = new JPanel();
  JLabel jLabel1 = new JLabel();
  FlowLayout flowLayout2 = new FlowLayout();
  JLabel jLabel2 = new JLabel();
  JLabel jLabel3 = new JLabel();
  JLabel jLabel4 = new JLabel();
  JLabel jLabel5 = new JLabel();
  JLabel lblProdCnt = new JLabel();
  JLabel lblConsCnt = new JLabel();
  JLabel lblItemCnt = new JLabel();
  JLabel lblCurrPolicy = new JLabel();
  JLabel lblNewPolicy = new JLabel();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  BorderLayout borderLayout1 = new BorderLayout();
  BorderLayout borderLayout2 = new BorderLayout();
  JButton btnWriteFile = new JButton();
  JButton btnReadFile = new JButton();
  FlowLayout flowLayout1 = new FlowLayout();
  /**
   *
   */
  private String[] policyNames = { "Adv_PreferFIFO",
                                   "Adv_PreferGet",
                                   "Adv_PreferPut" };
  /**
   *
   */
  private PolicyDataSet myPolicyDataSet = null;
  /**
   *
   */
  private PolicyChooser myPolicyChooser = null;
  /**
   *
   */
  public AgentDialog(Frame frame, String title, boolean modal) {
    super(frame, title, modal);
    try {
      jbInit();
      pack();
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
    // Load all the scheduling policies
    AspectRepository ar = ProdConsApp.getAspectRepository();
    ar.addPolicy( policyNames[0] );
    ar.addPolicy( policyNames[1] );
    ar.addPolicy( policyNames[2] );
  }

  /**
   *
   */
  public AgentDialog(PolicyDataSet pds) {
    this(null, "", false);
    myPolicyDataSet = pds;
    myPolicyChooser = new PolicyChooser(pds);
  }

  void jbInit() throws Exception {
    panel1.setLayout(borderLayout1);
    jPanel2.setBorder(BorderFactory.createEtchedBorder());
    jPanel2.setLayout(gridBagLayout1);
    jPanel3.setBorder(BorderFactory.createEtchedBorder());
    jPanel3.setLayout(flowLayout1);
    btnHide.setText("Hide");
    btnHide.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnHide_actionPerformed(e);
      }
    });
    btnDisable.setText("Disable");
    btnDisable.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnDisable_actionPerformed(e);
      }
    });
    this.setResizable(false);
    this.setTitle("Agent");
    this.getContentPane().setLayout(borderLayout2);
    jPanel1.setBorder(BorderFactory.createEtchedBorder());
    jPanel1.setLayout(flowLayout2);
    jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
    jLabel1.setText("Producers in Queue");
    jLabel2.setHorizontalAlignment(SwingConstants.RIGHT);
    jLabel2.setText("Consumers in Queue");
    jLabel3.setHorizontalAlignment(SwingConstants.RIGHT);
    jLabel3.setText("Items in Buffer");
    jLabel4.setHorizontalAlignment(SwingConstants.RIGHT);
    jLabel4.setText("Current Policy");
    jLabel5.setHorizontalAlignment(SwingConstants.RIGHT);
    jLabel5.setText(".");
    lblProdCnt.setForeground(Color.blue);
    lblProdCnt.setHorizontalAlignment(SwingConstants.LEFT);
    lblProdCnt.setHorizontalTextPosition(SwingConstants.LEFT);
    lblConsCnt.setForeground(Color.blue);
    lblConsCnt.setHorizontalAlignment(SwingConstants.LEFT);
    lblConsCnt.setHorizontalTextPosition(SwingConstants.LEFT);
    lblItemCnt.setForeground(Color.blue);
    lblItemCnt.setHorizontalAlignment(SwingConstants.LEFT);
    lblItemCnt.setHorizontalTextPosition(SwingConstants.LEFT);
    lblCurrPolicy.setForeground(Color.blue);
    lblCurrPolicy.setHorizontalAlignment(SwingConstants.LEFT);
    lblCurrPolicy.setHorizontalTextPosition(SwingConstants.LEFT);
    lblNewPolicy.setForeground(Color.red);
    lblNewPolicy.setHorizontalAlignment(SwingConstants.LEFT);
    lblNewPolicy.setHorizontalTextPosition(SwingConstants.LEFT);
    btnWriteFile.setText("Write to File");
    btnWriteFile.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnWriteFile_actionPerformed(e);
      }
    });
    btnReadFile.setText("Read from File");
    btnReadFile.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnReadFile_actionPerformed(e);
      }
    });
    getContentPane().add(panel1, BorderLayout.CENTER);
    panel1.add(jPanel2, BorderLayout.CENTER);
    jPanel2.add(jLabel2,  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 12, 0, 0), 41, 9));
    jPanel2.add(jLabel3,  new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(9, 12, 0, 0), 81, 9));
    jPanel2.add(jLabel4,  new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(10, 12, 0, 0), 84, 9));
    jPanel2.add(lblProdCnt,   new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(19, 0, 0, 95), 72, 29));
    jPanel2.add(lblConsCnt,   new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 0, 0, 95), 67, 29));
    jPanel2.add(lblItemCnt,   new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(8, 0, 0, 95), 76, 29));
    jPanel2.add(lblCurrPolicy,  new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(8, 0, 0, 95), 129, 29));
    jPanel2.add(lblNewPolicy,  new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 0, 54, 95), 129, 29));
    jPanel2.add(jLabel5,  new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(9, 12, 54, 0), 47, 9));
    jPanel2.add(jLabel1,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(19, 12, 0, 0), 50, 9));
    panel1.add(jPanel3, BorderLayout.SOUTH);
    jPanel3.add(btnReadFile, null);
    jPanel3.add(btnWriteFile, null);
    jPanel3.add(btnDisable, null);
    jPanel3.add(btnHide, null);
    panel1.add(jPanel1, BorderLayout.NORTH);
  }

  // implementation of the Observer interface --- public void update()

  private static int myCnt = 0;
  public void update(Observable o, Object arg) {
    HistoryQueue hq = (HistoryQueue) arg;
    int size = hq.size();

    if (size < 1) return;
    int curr_n_items = ((Integer) hq.getHistoryI().get(size - 1)).intValue();
    int curr_n_puts = ((Integer) hq.getHistoryP().get(size - 1)).intValue();
    int curr_n_gets = ((Integer) hq.getHistoryC().get(size - 1)).intValue();

    lblProdCnt.setText(hq.getHistoryP().get(size - 1).toString());
    lblConsCnt.setText(hq.getHistoryC().get(size - 1).toString());
    lblItemCnt.setText(hq.getHistoryI().get(size - 1).toString());
    lblCurrPolicy.setText(hq.getHistoryS().get(size - 1).toString());

    if (btnDisable.getText().equals("Enable")) return;
    //System.out.println("--> update() was called in AgentDialog.");
    myCnt++;
    if (myCnt > 10) {
      myCnt = 0;
      int x = myPolicyChooser.getBestPolicy(curr_n_items, curr_n_gets, curr_n_puts);
      AspectRepository ar = ProdConsApp.getAspectRepository();
      ar.changePolicy( policyNames[x] );
      lblNewPolicy.setText("");
    }
  }

  void btnHide_actionPerformed(ActionEvent e) {
    this.hide();
  }

  void btnDisable_actionPerformed(ActionEvent e) {
    if (btnDisable.getText().equals("Disable"))
      btnDisable.setText("Enable");
    else
      btnDisable.setText("Disable");
  }

  void btnReadFile_actionPerformed(ActionEvent e) {
  }

  void btnWriteFile_actionPerformed(ActionEvent e) {
    // Create a file dialog box for a new file to write data into.
    JFileChooser  fc = new JFileChooser ();
    fc.setDialogTitle("DWF - Policy Data Writer");

    // Display the dialog and wait for the user's response
    int returnVal = fc.showOpenDialog(this);
    if (returnVal != JFileChooser.APPROVE_OPTION) return;

    String myFileName = fc.getSelectedFile().getName();

    try {
        myPolicyDataSet.writeToFile(myFileName);
    } catch (Exception ex) {
        JOptionPane.showMessageDialog(this,
            "Failed to write data to a file " + myFileName + "\n" +
            "Error: " + ex.getLocalizedMessage());
        //ex.printStackTrace();
      return;
    }
  }
}
