import java.io.File;
import java.util.StringTokenizer;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;

/**
 * Title:        Producer Consumer - Dynamic Weaver Framework
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

/**
 * AspectDialog extends JDialog.
 * It provides user interface for adding and removing advices to/from the system.
 * When the dialog opens, all the advices currently registered on the system
 * are displayed in the listbox.
 * To add an advice, user selects the join points by selecting the check boxes,
 * and then presses the 'add' button. Then a FileChooser dialog will open.
 * By selecting an advice class file from the file list, the advice will
 * be registered to the system.
 * To delete advices, user simply highlights the advices on the listbox by clocking on them,
 * Then clicks the 'remove' button.
 */
public class AspectDialog extends JDialog {
  JPanel panel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  JPanel jPanel3 = new JPanel();
  JButton btnAdd = new JButton();
  JButton btnRemove = new JButton();
  JButton btnClose = new JButton();
  JLabel jLabel1 = new JLabel();
  Border border1;
  JPanel jPanel1 = new JPanel();
  JScrollPane jScrollPane1 = new JScrollPane();
  JList adviceList = new JList();
  GridBagLayout gridBagLayout1 = new GridBagLayout();

  // The default directory to display in the FileDialog
  File currDir = new File(System.getProperty("user.dir"));
  JPanel jPanel4 = new JPanel();
  JLabel jLabel2 = new JLabel();
  JCheckBox checkboxGet = new JCheckBox();
  JCheckBox checkboxPut = new JCheckBox();
  GridBagLayout gridBagLayout2 = new GridBagLayout();
  FlowLayout flowLayout1 = new FlowLayout();
  JPanel jPanel5 = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();

  public AspectDialog(Frame frame, String title, boolean modal) {
    super(frame, title, modal);
    try {
      jbInit();
      pack();
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
  }

  public AspectDialog() {
    this(null, "", false);
  }
  void jbInit() throws Exception {
    border1 = BorderFactory.createBevelBorder(BevelBorder.LOWERED,Color.white,Color.white,new Color(142, 142, 142),new Color(99, 99, 99));
    panel1.setLayout(borderLayout1);
    jPanel2.setBorder(BorderFactory.createEtchedBorder());
    jPanel2.setLayout(gridBagLayout1);
    jPanel3.setBorder(BorderFactory.createEtchedBorder());
    jPanel3.setLayout(flowLayout1);
    btnAdd.setText("Add");
    btnAdd.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnAdd_actionPerformed(e);
      }
    });
    btnRemove.setText("Remove");
    btnRemove.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnRemove_actionPerformed(e);
      }
    });
    btnClose.setText("Close");
    btnClose.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnClose_actionPerformed(e);
      }
    });
    jLabel1.setText("Registered Advices");
    jPanel1.setBorder(BorderFactory.createEtchedBorder());
    this.setResizable(false);
    this.setTitle("Other Aspects");
    jScrollPane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    jPanel4.setBorder(BorderFactory.createEtchedBorder());
    jPanel4.setLayout(gridBagLayout2);
    jLabel2.setText("Add to:");
    checkboxGet.setActionCommand("");
    checkboxGet.setSelected(true);
    checkboxGet.setText("get()");
    checkboxPut.setActionCommand("");
    checkboxPut.setSelected(true);
    checkboxPut.setText("put()");
    jPanel4.add(jLabel2, new GridBagConstraints(0, 0, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 2, 0, 41), 62, -6));
    jPanel4.add(checkboxGet, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 2, 1, 0), 26, -13));
    jPanel4.add(checkboxPut, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 1, 1), 11, -12));
    jPanel3.add(jPanel4, null);
    jPanel3.add(jPanel5, null);
    jPanel3.add(btnAdd, null);
    jPanel3.add(btnRemove, null);
    jPanel3.add(btnClose, null);
    getContentPane().add(panel1);
    panel1.add(jPanel2, BorderLayout.CENTER);
    jPanel2.add(jLabel1,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 6, 0, 177), 105, 3));
    jPanel2.add(jScrollPane1,  new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 6, 8, 6), 107, 74));
    jScrollPane1.getViewport().add(adviceList, null);
    panel1.add(jPanel3, BorderLayout.SOUTH);
    panel1.add(jPanel1, BorderLayout.NORTH);
  }


  public void show() {
    super.show();

    AspectRepository ar = ProdConsApp.getAspectRepository();
    adviceList.setListData(ar.getAllPointCutInfos());
  }

  void btnClose_actionPerformed(ActionEvent e) {
    this.hide();
  }

  void btnRemove_actionPerformed(ActionEvent e) {
    if (adviceList.getModel().getSize() == 0) {
      JOptionPane.showMessageDialog(this, "No items in the list.");
      return;
    }
    if (adviceList.isSelectionEmpty()) {
      JOptionPane.showMessageDialog(this, "First select advices to be removed.");
      return;
    }

    // remove all the selected advices
    Object [] advices = adviceList.getSelectedValues();
    AspectRepository ar = ProdConsApp.getAspectRepository();

    for (int i = 0; i < advices.length; i++) {
      StringTokenizer st = new StringTokenizer((String) advices[i], ", ");
      String aspectName = st.nextToken();
      String pointcutName = st.nextToken();
      String methodName = st.nextToken();
      ar.removeAspect(aspectName, pointcutName, methodName);
    }
    show();
  }

  void btnAdd_actionPerformed(ActionEvent e) {
    if (!checkboxGet.isSelected() && !checkboxPut.isSelected()) {
      JOptionPane.showMessageDialog(this, "Select component's methods first.");
      return;
    }
    // Create a file dialog box to prompt for a new file to load.
    JFileChooser  fc = new JFileChooser ();
    fc.setDialogTitle("DWF - Advice Loader");
    fc.setCurrentDirectory(currDir);

    // Display the dialog and wait for the user's response
    int returnVal = fc.showOpenDialog(this);

    // Remember new default direcotry
    currDir = fc.getCurrentDirectory();

    if (returnVal != JFileChooser.APPROVE_OPTION) return;

    String myFileName = fc.getSelectedFile().getName();
    int idx = myFileName.indexOf(".class", 0);
    if (idx < 0) {
      JOptionPane.showMessageDialog(this, myFileName + " is not a class file.");
      return;
    }

    String myClassName = myFileName.substring(0, idx);  //  without extension
    AspectRepository ar = ProdConsApp.getAspectRepository();
    try {
        Class myClass = Class.forName(myClassName);
        AdviceIF myNewAdvice = (AdviceIF) myClass.newInstance();
        // advice provides the aspect name
        String aspectName = myNewAdvice.getAspectName();

        // Add the loaded advice to the component's methods selected.
        if (checkboxGet.isSelected())
          ar.addAspect(aspectName, "DEFAULT", "get", myNewAdvice);
        if (checkboxPut.isSelected())
          ar.addAspect(aspectName, "DEFAULT", "put", myNewAdvice);

        this.show();
    } catch (Exception ex) {
        JOptionPane.showMessageDialog(this,
            "Failed to add a new advice " + myClassName + "\n" +
            "Error: " + ex.getLocalizedMessage());
        //ex.printStackTrace();
      return;
    }
  }
}
