import java.lang.reflect.*;

/**
 * Dynamic Weaver or Dynamic Proxy
 *
 */
public class AspectWeaver implements InvocationHandler {
  private AspectRepository _ar;
  private Object _buf;

  public static Object newInstance(Object buf, AspectRepository ar) {
      return Proxy.newProxyInstance(buf.getClass().getClassLoader(),
                                buf.getClass().getInterfaces(),
                                new AspectWeaver(buf, ar));
  }

  private AspectWeaver(Object buf, AspectRepository ar) {
      _ar = ar;
      _buf = buf;
  }

  /**
   * Whenever a client of the shared object (e.g. bounded buffer) tries to
   * execute one of the method of the shared object, this invoke() method
   * will be executed.
   * First, the AspectRepository's beforeAdvice() will be executed.
   * If it returns RESUME, the actual invocation of the shared object's method will be
   * performed. After that, AspectRepository's afterAdvice() will be excuted.
   */
  public Object invoke(Object proxy, Method m, Object[] args) throws Throwable {
    Object result = null;
    try {
      _ar.beforeAdvice(_buf, m, args);
      //System.out.println("RESUME " + m.getName());
      result = m.invoke(_buf, args);
      _ar.afterAdvice(_buf, m, args, result);
    } catch (InvocationTargetException e) {
      throw e.getTargetException();
    } /*catch (Exception e) {
        throw new RuntimeException("unexpected invocation exception: " +
        e.getMessage()+"\n");
      }*/
    return result;
  }
}

