import java.util.Vector;

/**
 * An instance of HistoryQueue will be maintained inside the AspectRepository.
 * This is not an inherent part of the DWF. This is used only for the
 * visualization of the queues and buffer.
 */
public class HistoryQueue {
  private Vector _history_P = new Vector(); /* producer */
  private Vector _history_C = new Vector(); /* consumer */
  private Vector _history_I = new Vector(); /* item count */
  private Vector _history_S = new Vector(); /* scheduling policies */

  HistoryQueue() { }

  public synchronized boolean isEmpty() {
    // Returns true if the queue is empty.
    return _history_P.size() == 0;
  }

  public synchronized void enqueue(int p_cnt, // num of producers
                               int c_cnt, // num of consumers
                               int i_cnt, // num of items
                               String policyName // policy
                               ) {
    if (_history_P.size() > 1000) dequeue();

    _history_P.add(new Integer(p_cnt));
    _history_C.add(new Integer(c_cnt));
    _history_I.add(new Integer(i_cnt));
    _history_S.add(policyName);
  }

  public synchronized void dequeue() {
    if (_history_P.size() == 0) return;
    _history_P.remove(0);
    _history_C.remove(0);
    _history_I.remove(0);
    _history_S.remove(0);
  }

  public synchronized int size() {
    // Returns the size of the queue.
    return _history_P.size();
  }

  public synchronized void clear() {
    //
    _history_P.clear();
    _history_C.clear();
    _history_I.clear();
    _history_S.clear();
  }

  public int getPcnt(int index) { return ((Integer) _history_P.get(index)).intValue(); }
  public int getCcnt(int index) { return ((Integer) _history_C.get(index)).intValue(); }
  public int getIcnt(int index) { return ((Integer) _history_I.get(index)).intValue(); }
  public String getPolicy(int index) { return (String) _history_S.get(index); }

  public Vector getHistoryP() { return _history_P; }
  public Vector getHistoryC() { return _history_C; }
  public Vector getHistoryI() { return _history_I; }
  public Vector getHistoryS() { return _history_S; }
}


