import java.util.Vector;
import java.awt.*;
import javax.swing.JPanel;

/**
 * Title:        Producer Consumer - Dynamic Weaver Framework
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

/**
 * We need three line graphs - one for the number of tasks in the get queue,
 * one for the number of tasks in the put queue, and finally one for the number of
 * items in the buffer. All of the three graphs are to be created using one class.
 * Because of the frequent update of the data to be displayed, the graph drawing routine
 * itself need to be efficient enough to prevent flickering from happening.
 * Overriding the paint() method gives the answer to this problem.
 */
public class MyLineGraphPanel extends JPanel {
  private Vector _vH; /* contains height info */
  private Vector _vC; /* contains color info */
  private int _maxH;  /* max height value possible in the vector */

  public MyLineGraphPanel() {
    super();
  }

  public void paint(Graphics g) {
    final float heightRatio = (float) ( 1.0 * this.getHeight() / _maxH );
    int iStart = 0;             // start index
    int iEnd;                   // end index
    int old_x = 0, old_y = this.getHeight();
    int new_x = 0, new_y = this.getHeight();

    super.paint(g);

    if (_vH == null || _vC == null) return;

    if (this.getWidth() >= _vH.size()) {  // if the panel is wide enough
        iStart = 0; iEnd = _vH.size() - 1;   // display all the vector
    } else {                             // if the panel is narrow
        iStart = _vH.size() - this.getWidth(); iEnd = _vH.size() - 1;    // truncate the earlier part
    }

    new_y = this.getHeight() - (int) ( ((Integer) _vH.get(iStart)).intValue() * heightRatio );
    for (int i = iStart; i < iEnd ; i++) {
        String pName = _vC.get(i).toString();
//        System.out.println(pName);
        if (pName.equals("PreferFIFO"))   g.setColor(Color.orange);    // fifo
        else if (pName.equals("PreferGet")) g.setColor(Color.green);
        else if (pName.equals("PreferPut")) g.setColor(Color.red);
        else g.setColor(Color.white);

        g.drawLine(old_x, old_y, new_x, new_y);
        old_x = new_x;
        old_y = new_y;
        new_x++;
        new_y = this.getHeight() - (int) ( ((Integer) _vH.get(i)).intValue() * heightRatio );
    }
  }

  public void redraw(Vector vHeights, Vector vColors, int maxHeight) {
  // redraw the chart using the given information about height and color
    _vH = vHeights;
    _vC = vColors;
    _maxH = maxHeight;
    repaint();
  }
}
