import java.lang.reflect.*;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Pointcut has a name and a table for joinpoints associated with it.
 *
 */
public class Pointcut {
  private String _pointcut_name;
  public Hashtable _advices = new Hashtable();

  /**
   * Constructor.
   * Sets the name of the pointcut to a given name.
   */
  public Pointcut(String name) {
    _pointcut_name = name;
  }

  /**
   * Returns the name of the pointcut.
   * Same as getName().
   */
  public String toString() {
    return _pointcut_name;
  }

  /**
   * Returns the name of the pointcut.
   * Same as toString().
   */
  public String getName() {
    return _pointcut_name;
  }

  /**
   * Returns the advice for the given join point
   * from the advices table.
   * Returns null if the advice or join point does not exist.
   */
  public AdviceIF getAdvice(String jp_name) {
     return (AdviceIF) _advices.get(jp_name);
  }

  /**
   * An advice for the given jp_name is
   * added to the current pointcut object.
   *
   * If the advice for the given jp_name is already there,
   * it will be replaced with the new one.
   */
  public void addAdvice(String jp_name, AdviceIF advice){
    if (_advices.containsKey(jp_name)) {
      _advices.remove(jp_name);
    }
    _advices.put(jp_name, advice);
  }

  /**
   * Removes the advice of the given jp_name from the advices table.
   */
  public void removeAdvice(String jp_name) {
    if (_advices.containsKey(jp_name)) {
      _advices.remove(jp_name);
    }
  }

  /**
   * Invokes the beforeAdvice() of the relevant advice.
   */
  public int beforeAdvice(Object obj, Method m, Object[] args){
    AdviceIF adv = (AdviceIF) _advices.get(m.getName());

    if (adv == null)
      adv = (AdviceIF) _advices.get("DEFAULT");

    if (adv == null)
      return AspectRepository.RESUME;

    return adv.beforeAdvice(obj, m, args);
  }

  /**
   * Invokes the afterAdvice() of the relevant advice.
   */
  public Object afterAdvice(Object obj, Method m, Object[] args, Object result){
    AdviceIF adv = (AdviceIF) _advices.get(m.getName());

    if (adv == null)
      adv = (AdviceIF) _advices.get("DEFAULT");

    if (adv != null)
      return adv.afterAdvice(obj, m, args, result);

    return null;
  }
}
