import java.util.Vector;
import java.util.Arrays;
import java.lang.Math;

/**
 * Agent uses PolicyChooser to determine the best policy
 * based on the training data and the Bayesian algorithm.
 *
 */
public class PolicyChooser {
  /**
   *
   */
  static final int POLICY_FIFO = 0;
  static final int POLICY_PGET = 1;
  static final int POLICY_PPUT = 2;

  private int FIFO_item_arr [], FIFO_gets_arr [], FIFO_puts_arr [];
  private int PGET_item_arr [], PGET_gets_arr [], PGET_puts_arr [];
  private int PPUT_item_arr [], PPUT_gets_arr [], PPUT_puts_arr [];

  private int    FIFO_cnt,  PGET_cnt,  PPUT_cnt;
  private double FIFO_freq, PGET_freq, PPUT_freq;

  // means
  private double FIFO_item_avg, FIFO_gets_avg, FIFO_puts_avg;
  private double PGET_item_avg, PGET_gets_avg, PGET_puts_avg;
  private double PPUT_item_avg, PPUT_gets_avg, PPUT_puts_avg;

  // stdevs
  private double FIFO_item_std, FIFO_gets_std, FIFO_puts_std;
  private double PGET_item_std, PGET_gets_std, PGET_puts_std;
  private double PPUT_item_std, PPUT_gets_std, PPUT_puts_std;

  // partial probabilities
  private double FIFO_item_p, FIFO_gets_p, FIFO_puts_p;
  private double PGET_item_p, PGET_gets_p, PGET_puts_p;
  private double PPUT_item_p, PPUT_gets_p, PPUT_puts_p;

  //
  private double FIFO_prob_temp;
  private double PGET_prob_temp;
  private double PPUT_prob_temp;

  // Final Probabilities
  private double FIFO_prob_final;
  private double PGET_prob_final;
  private double PPUT_prob_final;

  // Constructors
  /**
   *
   */
  public PolicyChooser(int arrFIFO[][], int arrPGET[][], int arrPPUT[][]) {
    setData(arrFIFO, arrPGET, arrPPUT);
  }
  /**
   *
   */
  public PolicyChooser(Vector v_FF, Vector v_PG, Vector v_PP) {
    setData(v_FF, v_PG, v_PP);
  }
  /**
   *
   */
  public PolicyChooser(PolicyDataSet pds) {
    setData(pds.getVector(PolicyChooser.POLICY_FIFO),
            pds.getVector(PolicyChooser.POLICY_PGET),
            pds.getVector(PolicyChooser.POLICY_PPUT));
  }
  /**
   *
   */
  public void setData(int arrFIFO[][], int arrPGET[][], int arrPPUT[][]) {
    int i;

    //--------------------------------------------
    FIFO_item_arr = new int[arrFIFO.length];
    FIFO_gets_arr = new int[arrFIFO.length];
    FIFO_puts_arr = new int[arrFIFO.length];

    for (i = 0; i < arrFIFO.length; i++) {
      FIFO_item_arr[i] = arrFIFO[i][0];
      FIFO_gets_arr[i] = arrFIFO[i][1];
      FIFO_puts_arr[i] = arrFIFO[i][2];
    }

    //--------------------------------------------
    PGET_item_arr = new int[arrPGET.length];
    PGET_gets_arr = new int[arrPGET.length];
    PGET_puts_arr = new int[arrPGET.length];

    for (i = 0; i < arrPGET.length; i++) {
      PGET_item_arr[i] = arrPGET[i][0];
      PGET_gets_arr[i] = arrPGET[i][1];
      PGET_puts_arr[i] = arrPGET[i][2];
    }
    //--------------------------------------------
    PPUT_item_arr = new int[arrPPUT.length];
    PPUT_gets_arr = new int[arrPPUT.length];
    PPUT_puts_arr = new int[arrPPUT.length];

    for (i = 0; i < arrPPUT.length; i++) {
      PPUT_item_arr[i] = arrPPUT[i][0];
      PPUT_gets_arr[i] = arrPPUT[i][1];
      PPUT_puts_arr[i] = arrPPUT[i][2];
    }

    calcStats();
  }

  /**
   *
   */
  public void setData(Vector v_FF, Vector v_PG, Vector v_PP) {
    PolicyDataRec myRec = null;
    int i;

    //--------------------------------------------
    FIFO_item_arr = new int[v_FF.size()];
    FIFO_gets_arr = new int[v_FF.size()];
    FIFO_puts_arr = new int[v_FF.size()];

    for (i = 0; i < v_FF.size(); i++) {
      myRec = (PolicyDataRec) v_FF.get(i);
      FIFO_item_arr[i] = myRec.i_cnt;
      FIFO_gets_arr[i] = myRec.g_cnt;
      FIFO_puts_arr[i] = myRec.p_cnt;
      System.out.println("FIFO : item = " + FIFO_item_arr[i] +
                               " gets = " + FIFO_gets_arr[i] +
                               " puts = " + FIFO_puts_arr[i]);
    }

    //--------------------------------------------
    PGET_item_arr = new int[v_PG.size()];
    PGET_gets_arr = new int[v_PG.size()];
    PGET_puts_arr = new int[v_PG.size()];

    for (i = 0; i < v_PG.size(); i++) {
      myRec = (PolicyDataRec) v_PG.get(i);
      PGET_item_arr[i] = myRec.i_cnt;
      PGET_gets_arr[i] = myRec.g_cnt;
      PGET_puts_arr[i] = myRec.p_cnt;
    }

    //--------------------------------------------
    PPUT_item_arr = new int[v_PP.size()];
    PPUT_gets_arr = new int[v_PP.size()];
    PPUT_puts_arr = new int[v_PP.size()];

    for (i = 0; i < v_PP.size(); i++) {
      myRec = (PolicyDataRec) v_PP.get(i);
      PPUT_item_arr[i] = myRec.i_cnt;
      PPUT_gets_arr[i] = myRec.g_cnt;
      PPUT_puts_arr[i] = myRec.p_cnt;
    }

    calcStats();
  }



  /**
   *
   */
  public int getBestPolicy(int item, int gets, int puts) {
  /**
   * item is the current number of items in the buffer
   * gets is the current number of tasks in the get queue
   * puts is the curretn number of tasks in the put queue
   *
   * Returns PolicyChooser.POLICY_FIFO if the FIFO is the best policy,
   *         PolicyChooser.POLICY_PGET if the Preference to GET is the best policy,
   *         PolicyChooser.POLICY_PPUT if the Preference to PUT is the best policy.
   */
    double power;

    // FIFO
    power = -0.5 * (item - FIFO_item_avg) * (item - FIFO_item_avg)/(FIFO_item_std * FIFO_item_std);
    FIFO_item_p = (0.39894228 / FIFO_item_std) * Math.exp(power);

    power = -0.5 * (puts - FIFO_puts_avg) * (puts - FIFO_puts_avg)/(FIFO_puts_std * FIFO_puts_std);
    FIFO_puts_p = (0.39894228 / FIFO_puts_std) * Math.exp(power);

    power = -0.5 * (gets - FIFO_gets_avg) * (gets - FIFO_gets_avg)/(FIFO_gets_std * FIFO_gets_std);
    FIFO_gets_p = (0.39894228 / FIFO_gets_std) * Math.exp(power);

    // PPUT
    power = -0.5 * (item - PPUT_item_avg) * (item - PPUT_item_avg)/(PPUT_item_std * PPUT_item_std);
    PPUT_item_p = (0.39894228 / PPUT_item_std) * Math.exp(power);

    power = -0.5 * (puts - PPUT_puts_avg) * (puts - PPUT_puts_avg)/(PPUT_puts_std * PPUT_puts_std);
    PPUT_puts_p = (0.39894228 / PPUT_puts_std) * Math.exp(power);

    power = -0.5 * (gets - PPUT_gets_avg) * (gets - PPUT_gets_avg)/(PPUT_gets_std * PPUT_gets_std);
    PPUT_gets_p = (0.39894228 / PPUT_gets_std) * Math.exp(power);

    // PGET
    power = -0.5 * (item - PGET_item_avg) * (item - PGET_item_avg)/(PGET_item_std * PGET_item_std);
    PGET_item_p = (0.39894228 / PGET_item_std) * Math.exp(power);

    power = -0.5 * (puts - PGET_puts_avg) * (puts - PGET_puts_avg)/(PGET_puts_std * PGET_puts_std);
    PGET_puts_p = (0.39894228 / PGET_puts_std) * Math.exp(power);

    power = -0.5 * (gets - PGET_gets_avg) * (gets - PGET_gets_avg)/(PGET_gets_std * PGET_gets_std);
    PGET_gets_p = (0.39894228 / PGET_gets_std) * Math.exp(power);

    System.out.println("------------------------- partial probabilities");
    System.out.println("FIFO_item_p : " + FIFO_item_p);
    System.out.println("FIFO_gets_p : " + FIFO_gets_p);
    System.out.println("FIFO_puts_p : " + FIFO_puts_p);

    System.out.println("---");
    System.out.println("PGET_item_p : " + PGET_item_p);
    System.out.println("PGET_gets_p : " + PGET_gets_p);
    System.out.println("PGET_puts_p : " + PGET_puts_p);

    System.out.println("---");
    System.out.println("PPUT_item_p : " + PPUT_item_p);
    System.out.println("PPUT_gets_p : " + PPUT_gets_p);
    System.out.println("PPUT_puts_p : " + PPUT_puts_p);

    //-----------------------
    FIFO_prob_temp = FIFO_freq * FIFO_item_p * FIFO_gets_p * FIFO_puts_p;
    PPUT_prob_temp = PPUT_freq * PPUT_item_p * PPUT_gets_p * PPUT_puts_p;
    PGET_prob_temp = PGET_freq * PGET_item_p * PGET_gets_p * PGET_puts_p;

    double tmp = FIFO_prob_temp + PPUT_prob_temp + PGET_prob_temp;

    FIFO_prob_final = FIFO_prob_temp / tmp;
    PPUT_prob_final = PPUT_prob_temp / tmp;
    PGET_prob_final = PGET_prob_temp / tmp;

    System.out.println("------------------------- final probabilities");
    System.out.println("FIFO_prob_final : " + FIFO_prob_final);
    System.out.println("PGET_prob_final : " + PGET_prob_final);
    System.out.println("PPUT_prob_final : " + PPUT_prob_final);

    if (FIFO_prob_final >= PPUT_prob_final && FIFO_prob_final >= PGET_prob_final)
      return POLICY_FIFO;

    if (PPUT_prob_final >= FIFO_prob_final && PPUT_prob_final >= PGET_prob_final)
      return POLICY_PPUT;

    return POLICY_PGET;
  }


  private void calcStats() {
  /**
   *    calculate means, stdevs etc fro the initial training data.
   */

    //----- number of items
    FIFO_cnt = FIFO_item_arr.length;
    PGET_cnt = PGET_item_arr.length;
    PPUT_cnt = PPUT_item_arr.length;

    System.out.println("------------------------- counts");
    System.out.println("FIFO_cnt : " + FIFO_cnt);
    System.out.println("PGET_cnt : " + PGET_cnt);
    System.out.println("PPUT_cnt : " + PPUT_cnt);

    //----- frequencies
    FIFO_freq = 1.0 * FIFO_cnt / (FIFO_cnt + PGET_cnt + PPUT_cnt);
    PGET_freq = 1.0 * PGET_cnt / (FIFO_cnt + PGET_cnt + PPUT_cnt);
    PPUT_freq = 1.0 * PPUT_cnt / (FIFO_cnt + PGET_cnt + PPUT_cnt);

    System.out.println("------------------------- frequencies");
    System.out.println("FIFO_freq : " + FIFO_freq);
    System.out.println("PGET_freq : " + PGET_freq);
    System.out.println("PPUT_freq : " + PPUT_freq);

    //----------- averages
    FIFO_item_avg = getAverage(FIFO_item_arr);
    FIFO_puts_avg = getAverage(FIFO_puts_arr);
    FIFO_gets_avg = getAverage(FIFO_gets_arr);

    PGET_item_avg = getAverage(PGET_item_arr);
    PGET_puts_avg = getAverage(PGET_puts_arr);
    PGET_gets_avg = getAverage(PGET_gets_arr);

    PPUT_item_avg = getAverage(PPUT_item_arr);
    PPUT_puts_avg = getAverage(PPUT_puts_arr);
    PPUT_gets_avg = getAverage(PPUT_gets_arr);

    System.out.println("------------------------- averages");
    System.out.println("FIFO_item_avg : " + FIFO_item_avg);
    System.out.println("FIFO_gets_avg : " + FIFO_gets_avg);
    System.out.println("FIFO_puts_avg : " + FIFO_puts_avg);

    System.out.println("---");
    System.out.println("PGET_item_avg : " + PGET_item_avg);
    System.out.println("PGET_gets_avg : " + PGET_gets_avg);
    System.out.println("PGET_puts_avg : " + PGET_puts_avg);

    System.out.println("---");
    System.out.println("PPUT_item_avg : " + PPUT_item_avg);
    System.out.println("PPUT_gets_avg : " + PPUT_gets_avg);
    System.out.println("PPUT_puts_avg : " + PPUT_puts_avg);

    //----------- standard deviations
    FIFO_item_std = getStdev(FIFO_item_arr);
    FIFO_puts_std = getStdev(FIFO_puts_arr);
    FIFO_gets_std = getStdev(FIFO_gets_arr);

    PGET_item_std = getStdev(PGET_item_arr);
    PGET_puts_std = getStdev(PGET_puts_arr);
    PGET_gets_std = getStdev(PGET_gets_arr);

    PPUT_item_std = getStdev(PPUT_item_arr);
    PPUT_puts_std = getStdev(PPUT_puts_arr);
    PPUT_gets_std = getStdev(PPUT_gets_arr);

    System.out.println("------------------------- standard deviations");
    System.out.println("FIFO_item_std : " + FIFO_item_std);
    System.out.println("FIFO_gets_std : " + FIFO_gets_std);
    System.out.println("FIFO_puts_std : " + FIFO_puts_std);

    System.out.println("---");
    System.out.println("PGET_item_std : " + PGET_item_std);
    System.out.println("PGET_gets_std : " + PGET_gets_std);
    System.out.println("PGET_puts_std : " + PGET_puts_std);

    System.out.println("---");
    System.out.println("PPUT_item_std : " + PPUT_item_std);
    System.out.println("PPUT_gets_std : " + PPUT_gets_std);
    System.out.println("PPUT_puts_std : " + PPUT_puts_std);
  }


  private double getAverage(int arr[]) {
  /**
   * Returns the average of the integer values contained in the integer array
   */
    int i, sum = 0;

    if (arr.length == 0) return 0.0;
    for( i = 0; i < arr.length; i++) sum += arr[i];
    return (sum * 1.0) / arr.length;
  }

  private double getStdev(int arr[]) {
  /**
   * Returns the standard deviation of the integer values contained in the integer array
   */
    int i, sum = 0;
    double avg;

    if (arr.length <= 1) return 0.0;

    for( i = 0; i < arr.length; i++) sum += arr[i];
    avg = (sum * 1.0) / arr.length;

    int sum_sq_diff = 0;

    for( i = 0; i < arr.length; i++) {
      sum_sq_diff += (arr[i] - avg) * (arr[i] - avg);
    }
    return Math.sqrt( 1.0 * sum_sq_diff / (arr.length - 1));
  }
}
