import java.util.Vector;
import java.io.*;
import javax.swing.JOptionPane;

/**
 * Title:        Producer Consumer - Dynamic Weaver Framework
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

 /**
  * Represents a set of policy data
  *
  */
public class PolicyDataSet {
  private Vector v_FF = null;  // First In First Out
  private Vector v_PG = null;  // Preference to Get
  private Vector v_PP = null;  // Preference to Put

  public PolicyDataSet() {
    clear();
  }

  public void clear() {
    v_FF = new Vector();
    v_PG = new Vector();
    v_PP = new Vector();
  }

  public Vector getVector(int policyCode) {
    switch (policyCode) {
    case PolicyChooser.POLICY_FIFO: return v_FF;
    case PolicyChooser.POLICY_PGET: return v_PG;
    case PolicyChooser.POLICY_PPUT: return v_PP;
    default: return null;
    }
  }

  public boolean addRec (int policyCode, PolicyDataRec rec) {
    switch (policyCode) {
    case PolicyChooser.POLICY_FIFO: return v_FF.add(rec);
    case PolicyChooser.POLICY_PGET: return v_PG.add(rec);
    case PolicyChooser.POLICY_PPUT: return v_PP.add(rec);
    default: return false;
    }
  }

  public boolean readFromFile(String filename) {
     clear();
     File in_file = new File(filename);
     FileInputStream in_stream = null;
     int ii, gg, pp;

     try {
       in_stream = new FileInputStream(in_file);
       Reader r = new BufferedReader(new InputStreamReader(in_stream));
       StreamTokenizer st = new StreamTokenizer(r);
       st.eolIsSignificant(false);
       st.slashSlashComments(true);
       st.slashSlashComments(true);
       st.whitespaceChars('{', '}');
       st.whitespaceChars(',', ',');

       System.out.println("----- Reading Policy Data from File " + filename );

       ii = gg = pp = -1;
       while (st.nextToken() != StreamTokenizer.TT_EOF) {
         PolicyDataRec myRec = new PolicyDataRec(-1, -1, -1);

         switch (st.ttype) {
         case StreamTokenizer.TT_NUMBER:
              System.out.print( (int) (st.nval) + ",  ");

              if (ii < 0) {
                ii = (int) (st.nval);
              }
              else if (gg < 0) {
                gg = (int) (st.nval);
              }
              else if (pp < 0) {
                pp = (int) (st.nval);
              }

              break;
         case StreamTokenizer.TT_WORD:
              System.out.println(st.sval);

              if (st.sval.equals("FIFO"))
                v_FF.add(new PolicyDataRec(ii, gg, pp));
              else if (st.sval.equals("PG"))
                v_PG.add(new PolicyDataRec(ii, gg, pp));
              else if (st.sval.equals("PP"))
                v_PP.add(new PolicyDataRec(ii, gg, pp));

              ii = gg = pp = -1;
              break;
         default: break;
         }
       }
       System.out.println("----- End of Policy Data");
     }
     catch (IOException e) {
        JOptionPane.showMessageDialog(null,
            "Error reading file :" + filename + "\n");
     }
     finally {
       if (in_stream != null)
         try {in_stream.close(); }
         catch (IOException e) {;}
     }

     return true;
  }

  /**
   *
   */
  public boolean writeToFile(String filename) {
     File o_file = new File(filename);

     try {
       PrintWriter w = new PrintWriter(new FileWriter(o_file));
       PolicyDataRec myRec = null;
       int i;
       System.out.println("----- Writing Policy Data to File " + filename );

       for (i = 0; i < v_FF.size(); i++) {
         myRec = (PolicyDataRec) v_FF.get(i);
         w.println(myRec.i_cnt + ", " + myRec.p_cnt + ", " +
                   myRec.g_cnt + ", " + "FIFO");
       }

       for (i = 0; i < v_PG.size(); i++) {
         myRec = (PolicyDataRec) v_PG.get(i);
         w.println(myRec.i_cnt + ", " + myRec.p_cnt + ", " +
                   myRec.g_cnt + ", " + "PG");
       }

       for (i = 0; i < v_PP.size(); i++) {
         myRec = (PolicyDataRec) v_PP.get(i);
         w.println(myRec.i_cnt + ", " + myRec.p_cnt + ", " +
                   myRec.g_cnt + ", " + "PP");
       }

       w.close();
       System.out.println("----- Finished Writing Policy Data");;
     }
     catch (IOException e) {
        JOptionPane.showMessageDialog(null,
            "Error writing file :" + filename + "\n");
     }

    return true;
  }
}
