import java.lang.Thread;
import java.util.Date;
import java.lang.reflect.Method;

/**
 * Abstract class for all the preference advice classes like
 * Adv_PreferFIFO, Adv_PreferGet, and Adv_PreferPut, etc.
 * All the scheduling advices must extend this abstract class.
 * By doing so, task queues defined here will be shared by all the scheduling advices.
 */

public abstract class Preference  {
  static protected TaskQueue _prodQ = new TaskQueue("put"); // Queue for producers
  static protected TaskQueue _consQ = new TaskQueue("get");; // Queue for consumers

  abstract public String getPolicyName();

  static public int getQsize(String method_name) {
  /**
   * do we need to use Hashtable for queues?
   */
    if (method_name.equals("put"))
      return _prodQ.size();
    else if (method_name.equals("get"))
      return _consQ.size();
    else
      return 0;
  }

  static public void enqueue(Task t) {
    if (t.getMethodName().equals("put"))
      _prodQ.enqueue(t);
    else if (t.getMethodName().equals("get"))
      _consQ.enqueue(t);
    else
      System.out.println("Error in enqueue");
  }

  static public void dequeue(Task t) {
    if (t.getMethodName().equals("put"))
      _prodQ.dequeue(t);
    else if (t.getMethodName().equals("get"))
      _consQ.dequeue(t);
    else
      System.out.println("Error in dequeue");
  }
}
