import javax.swing.UIManager;
import java.awt.*;

/**
 * main() is here.
 */
public class ProdConsApp {
  boolean packFrame = false;

  static private int myBufferSize = 65;
  static private int myProducerCnt = 65;
  static private int myConsumerCnt = 65;
  static private String myInputFileName = "PolicyTraining.txt";
  static AspectRepository aspectRepository = new AspectRepository();
  static BoundedBufferIF myBufferProxy = (BoundedBufferIF) AspectWeaver.newInstance (
                                         new BoundedBuffer(myBufferSize),
                                         aspectRepository);

  /**Construct the application*/
  public ProdConsApp() {
    ProdConsFrame frame = new ProdConsFrame();
    //Validate frames that have preset sizes
    //Pack frames that have useful preferred size info, e.g. from their layout
    if (packFrame) {
      frame.pack();
    }
    else {
      frame.validate();
    }

    // aspectRepository is observable. frame is the observer.
    aspectRepository.addObserver(frame);

    // initial loading of default advices --
    AdviceIF newAdvice;

    newAdvice = aspectRepository.loadAdvice("Adv_SyncGet");
    if (newAdvice != null) {
      aspectRepository.addAspect("SYNCH", "DEFAULT", "get", newAdvice);
    }

    newAdvice = aspectRepository.loadAdvice("Adv_SyncPut");
    if (newAdvice != null) {
      aspectRepository.addAspect("SYNCH", "DEFAULT", "put", newAdvice);
    }

    newAdvice = aspectRepository.loadAdvice(aspectRepository.getCurrPolicyName());
    if (newAdvice != null) {
      aspectRepository.addAspect("SCHED", "DEFAULT", "DEFAULT", newAdvice);
    }

    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = frame.getSize();
    if (frameSize.height > screenSize.height) {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width) {
      frameSize.width = screenSize.width;
    }
    frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    frame.setVisible(true);
  }

  /* simple accessors */
  public static AspectRepository getAspectRepository() { return aspectRepository; }
  public static BoundedBufferIF getMyBufferProxy() { return myBufferProxy; }
  public static int getMyBufferSize() { return myBufferSize; }
  public static int getMyProducerCnt() { return myProducerCnt; }
  public static int getMyConsumerCnt() { return myConsumerCnt; }
  public static String getMyInputFileName() { return "PolicyTraining.txt" /*myInputFileName*/; }

  public static void setMyBufferSize(int size) {
    myBufferSize = size;
    myBufferProxy = (BoundedBufferIF) AspectWeaver.newInstance (
                                      new BoundedBuffer(myBufferSize),
                                      aspectRepository);
  }
  public static void setMyProducerCnt(int cnt) { myProducerCnt = cnt; }
  public static void setMyConsumerCnt(int cnt) { myConsumerCnt = cnt; }
  public static void setMyInputFileName(String name) { myInputFileName = name; }

  /**Main method*/
  public static void main(String[] args) {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    new ProdConsApp();
  }
}

