import java.util.Date;

/**
 * Task is needed for synchronization and scheduling purposes.
 * A new task is created whenever a thread enters the AspectRepository's beforeAdvice() method.
 * The task expires when the corresponding task executes the AspectRepository's
 * afterAdvice() method.
 *
 * Only the minimal information for synchronization and scheduling is maintained
 * in this class.
 *  - Task id,
 *  - join point name (shared object's method name)
 *  - arrival time of the thread to the AspectRepository's beforeAdvice().
 */
public class Task {
  private String _task_id = null;
  private String _method_name = null; // name of the shared obj's method invoked by this task
  private Date _arrivalTime = null; // This will be used for FIFO ordering

  Task(String method_name) {
    _method_name = method_name;
    _task_id = Thread.currentThread().getName();
    _arrivalTime = new Date();
  }

  public String getName() { return _task_id; }
  public String getMethodName() { return _method_name; }
  public Date getArrivalTime() { return _arrivalTime; }
  public String toString() { return _task_id; }
}
