import java.util.Vector;
import java.util.Date;

/**
 * TaskQueues will only be statically defined in the
 * Preference abstract class.
 * Elements contained in  TaskQueue are Task objects.
 */
public class TaskQueue {
  // Task queue for the clients of bounded buffer
  // Borrowed the idea from the book - Doug Lea, p.310
  // Vector is synchronized.

  private Vector _tasks = new Vector();
  private String _name;

  TaskQueue(String name) { _name = name;  }

  public  boolean isEmpty() {
    // Returns true if the queue is empty.
    return _tasks.size() == 0;
  }

  public  void enqueue(Task task) {
    // Append the task t to the end of the queue.
    _tasks.add(task);
  }

  public  Task dequeue() {
    // Returns null if empty.
    if (_tasks.size() == 0) return null;
    return (Task) _tasks.remove(0);
  }

  public  boolean dequeue(Task t) {
    if (_tasks.size() == 0) return false;
    return _tasks.remove(t);
  }

  public  int size() {
    // Returns the size of the queue.
    return _tasks.size();
  }
  public String toString() {
      return _name;
  }

  public Date getArrivalTimeOfFirstTask() {
    if (_tasks.size() == 0) return null;
    return ((Task) _tasks.firstElement()).getArrivalTime();
  }
}
