
                     import java.sql.Connection;
                     import java.sql.DriverManager;
                     import java.sql.PreparedStatement;
                     import java.sql.ResultSet;
                     import java.sql.SQLException;
                     import java.sql.Statement;

                     public class AxionExample {
                         public static void main(String[] args) throws Exception {
                             Class.forName("org.axiondb.jdbc.AxionDriver");
                             AxionExample eg = new AxionExample();
                             eg.createTables("jdbc:axiondb:egdb");
                             eg.populateTables("jdbc:axiondb:egdb");
                             eg.printReport("jdbc:axiondb:egdb");
                         }

                         public void createTables(String uri) throws SQLException {
                             System.out.println("Creating tables");
                             Connection conn = null;
                             Statement stmt = null;
                             try {
                                conn = DriverManager.getConnection(uri);
                                stmt = conn.createStatement();
                                stmt.execute("create table PERSON ( PERSON_ID integer, " +
                                             "FIRSTNAME varchar, LASTNAME varchar )");
                                stmt.execute("create table PLACE ( PLACE_ID integer, " +
                                             "CITY varchar, STATE varchar )");
                                stmt.execute("create table PERSON_PLACE_MAP ( " +
                                             "PERSON_ID integer, PLACE_ID integer )");
                             } finally {
                                try { stmt.close(); } catch(Exception e) { }
                                try { conn.close(); } catch(Exception e) { }
                             }
                         }

                         public void populateTables(String uri) throws SQLException {
                             System.out.println("Populating tables");
                             populatePersonTable(uri);
                             populatePlaceTable(uri);
                             populateMapTable(uri);
                         }

                         public void populatePersonTable(String uri) throws SQLException {
                             Connection conn = null;
                             PreparedStatement stmt = null;
                             try {
                                conn = DriverManager.getConnection(uri);
                                stmt = conn.prepareStatement("insert into PERSON " +
                                                             "( PERSON_ID, FIRSTNAME, LASTNAME ) " + 
                                                             "values ( ?, ?, ?)");
                                {
                                   stmt.setInt(1,1);
                                   stmt.setString(2,"John");
                                   stmt.setString(3,"Doe");
                                   stmt.executeUpdate();
                                }
                                {
                                   stmt.setInt(1,2);
                                   stmt.setString(2,"Jane");
                                   stmt.setString(3,"Doe");
                                   stmt.executeUpdate();
                                }
                                {
                                   stmt.setInt(1,3);
                                   stmt.setString(2,"Jack");
                                   stmt.setString(3,"Nimble");
                                   stmt.executeUpdate();
                                }
                             } finally {
                                try { stmt.close(); } catch(Exception e) { }
                                try { conn.close(); } catch(Exception e) { }
                             }
                         }

                         public void populatePlaceTable(String uri) throws SQLException {
                             Connection conn = null;
                             Statement stmt = null;
                             try {
                                conn = DriverManager.getConnection(uri);
                                stmt = conn.createStatement();
                                stmt.executeUpdate("insert into PLACE " +
                                                   "( PLACE_ID, CITY, STATE ) " +
                                                   "values ( 1, 'Chicago', 'IL' )");
                                stmt.executeUpdate("insert into PLACE " +
                                                   "( PLACE_ID, CITY, STATE ) " +
                                                   "values ( 2, 'New York', 'NY' )");
                                stmt.executeUpdate("insert into PLACE " +
                                                   "( PLACE_ID, CITY, STATE ) " +
                                                   "values ( 3, 'Los Angeles', 'CA' )");
                             } finally {
                                try { stmt.close(); } catch(Exception e) { }
                                try { conn.close(); } catch(Exception e) { }
                             }
                         }

                         public void populateMapTable(String uri) throws SQLException {
                             Connection conn = null;
                             PreparedStatement stmt = null;
                             try {
                                conn = DriverManager.getConnection(uri);
                                stmt = conn.prepareStatement("insert into PERSON_PLACE_MAP " +
                                                             "( PERSON_ID, PLACE_ID ) " + 
                                                             "values ( ?, ?)");
                                {
                                   stmt.setInt(1,1);
                                   stmt.setInt(2,1);
                                   stmt.executeUpdate();
                                }
                                {
                                   stmt.setInt(1,2);
                                   stmt.setInt(2,1);
                                   stmt.executeUpdate();
                                }
                                {
                                   stmt.setInt(1,2);
                                   stmt.setInt(2,3);
                                   stmt.executeUpdate();
                                }
                                {
                                   stmt.setInt(1,3);
                                   stmt.setInt(2,2);
                                   stmt.executeUpdate();
                                }
                             } finally {
                                try { stmt.close(); } catch(Exception e) { }
                                try { conn.close(); } catch(Exception e) { }
                             }
                         }

                         public void printReport(String uri) throws SQLException {
                             System.out.println("Printing report");
                             String query = "select FIRSTNAME, LASTNAME, CITY, STATE " +
                                            "from PERSON A, PLACE B, PERSON_PLACE_MAP C " +
                                            "where A.PERSON_ID = C.PERSON_ID and " +
                                            "      B.PLACE_ID = C.PLACE_ID " +
                                            "order by LASTNAME, FIRSTNAME, STATE";

                             Connection conn = null;
                             Statement stmt = null;
                             ResultSet rset = null;
                             try {
                                conn = DriverManager.getConnection(uri);           
                                stmt = conn.createStatement();
                                rset = stmt.executeQuery(query);
                                System.out.println("FIRST\tLAST\tCITY\tSTATE");
                                while(rset.next()) {
                                  System.out.print(rset.getString(1));
                                  System.out.print("\t");
                                  System.out.print(rset.getString(2));
                                  System.out.print("\t");
                                  System.out.print(rset.getString(3));
                                  System.out.print("\t");
                                  System.out.println(rset.getString(4));
                                }        
                             } finally {
                                try { rset.close(); } catch(Exception e) { }
                                try { stmt.close(); } catch(Exception e) { }
                                try { conn.close(); } catch(Exception e) { }
                             }
                         }
                     }
