/**************************************************************************
/* This class represents a data objects for file information returned by
/* find $1 -type f -printf "%p\t%s\t%T@\t%u\t%g\t%n\t%m\n"
/*
/* Copyright (c) 2001 by Bernhard Bablok (mail@bablokb.de)
/*
/* This program is free software; you can redistribute it and/or modify
/* it under the terms of the GNU General Public License as published
/* by  the Free Software Foundation; either version 2 of the License or
/* (at your option) any later version.
/*
/* This program is distributed in the hope that it will be useful, but
/* WITHOUT ANY WARRANTY; without even the implied warranty of
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/* GNU General Public License for more details.
/*
/* You should have received a copy of the GNU General Public License
/* along with this program; see the file COPYING.  If not, write to
/* the Free Software Foundation Inc., 59 Temple Place - Suite 330,
/* Boston, MA  02111-1307 USA
/**************************************************************************/
//package de.bablokb.jdbc;

import java.util.*;
import java.sql.*;

/**
   This class represents a data objects for file information returned by
   <pre>
                            filename
                            |   size in bytes
                            |   |   modification time (seconds since 1970)
                            |   |   |    user name
                            |   |   |    |   group name
                            |   |   |    |   |   hard links
                            |   |   |    |   |   |   permission bits (octal)
                            |   |   |    |   |   |   |
   find $1 -type f -printf "%p\t%s\t%T@\t%u\t%g\t%n\t%m\n"
   </pre>

   @version $Revision: 1.10 $
   @author  $Author: Bablokb $
*/

public class FileInfo {


  public String    iFileName, iUserName, iGroupName, iPermBits;
  public int       iLinkCount, iSize;
  public Timestamp iModTime;
  
  //////////////////////////////////////////////////////////////////////////////

  /**
     Constructor. Parses a line returned by find command.
  */

  public FileInfo(String line) throws NumberFormatException {
    StringTokenizer t = new StringTokenizer(line,"\t\n");
    iFileName  = t.nextToken();
    iSize      = Integer.parseInt(t.nextToken());
    iModTime   = new Timestamp(1000*Long.parseLong(t.nextToken()));
    iUserName  = t.nextToken();
    iGroupName = t.nextToken();
    iLinkCount = Integer.parseInt(t.nextToken());
    iPermBits  = t.nextToken();
  }
  
  //////////////////////////////////////////////////////////////////////////////

  /**
     Convert object to a string representation suitable for a sql insert
     statement:
     <pre>
     INSERT INTO foo (col1,col2,...) VALUES(val1,val2,...)
     </pre>
  */

  public String toInsertString() {
    StringBuffer buf = new StringBuffer(300);
    buf.append("VALUES('");
    buf.append(iFileName);
    buf.append("',");
    buf.append(Integer.toString(iSize));
    buf.append(",'");
    buf.append(iModTime.toString());
    buf.append("','");
    buf.append(iUserName);
    buf.append("','");
    buf.append(iGroupName);
    buf.append("',");
    buf.append(Integer.toString(iLinkCount));
    buf.append(",'");
    buf.append(iPermBits);
    buf.append("')");
    return buf.toString();
  }

  //////////////////////////////////////////////////////////////////////////////

  /**
     Set IN parameters of prepared statement with object fields.
  */

  public void setParameter(PreparedStatement s) throws SQLException {
    s.setString(1,iFileName);
    s.setInt(2,iSize);
    s.setTimestamp(3,iModTime);
    s.setString(4,iUserName);
    s.setString(5,iGroupName);
    s.setInt(6,iLinkCount);
    s.setString(7,iPermBits);
    return;
  }
}
