import java.sql.*;
class Sample {

    static Connection con; // database connection object

    /** Main routine */
    public static void main(String args[]) throws Exception {

        // Load the JDBC-ODBC bridge driver 
        Class.forName ("sun.jdbc.odbc.JdbcOdbcDriver");

        // Open the database and display all rows in a table
        open();
        select();

        // Add a new row and show the result insert();
        select("WHERE EmployeeId = 100");

        // Update a row and commit the result
        update(true);
        select("WHERE EmployeeId = 100");

        // Update a row and rollback the result
        update(false);
        select("WHERE EmployeeId = 100");

        // Delete a row and show the result
        delete();
        select("WHERE EmployeeId = 100");

        // Close the database
        close();

    }

    /** Open a database connection */
    static void open() throws SQLException {

        // ODBC data source name 
        String dsn = "jdbc:odbc:Northwind"; 
        String user = "admin"; 
        String password = "";

        // Connect to the database 
        con = DriverManager.getConnection(dsn, user, password);

        // Shut off autocommit 
        con.setAutoCommit(false);

    }

    /** Commit all pending transactions and close the database connection */ 
    static void close() throws SQLException {

        con.commit(); 
        con.close();

    }

    /** Issue a SQL query with a WHERE clause */ 
    static void select(String whereClause) throws SQLException {

        Statement stmt; // SQL statement object 
        String query;   // SQL select string 
        ResultSet rs;   // SQL query results 
        boolean more;   // "more rows found" switch

        query = "SELECT EmployeeID, LastName, FirstName, Title "
              + "FROM Employees "
              + whereClause;

        stmt = con.createStatement();
        rs = stmt.executeQuery(query);

        // Check to see if any rows were read
        more = rs.next(); 
        if (!more) {

            System.out.println("No rows found."); 
            return;

        }

        // Loop through the rows retrieved from the query
        while (more) {

            System.out.println("ID: " + rs.getInt("EmployeeId")); 
            System.out.println("Name: " + rs.getString("FirstName") + " " + rs.getString("LastName")); 
            System.out.println("Title: " + rs.getString("Title")); 
            System.out.println(""); 
            more = rs.next();

        }

        rs.close(); 
        stmt.close();

    }

    /** Issue a SQL query without a WHERE clause */ 
    static void select() throws SQLException {

        select("");

    }

    /** Insert a new row */ 
    static void insert() throws SQLException {

        Statement stmt; // SQL statement object 
        String sql;     // SQL insert command 
        int rows;       // Number of rows inserted

        sql = "INSERT INTO Employees " 
            + "VALUES (100, 'Alexander', 'Jason', 'Baseball Manager', " 
            + "NULL, NULL, NULL, NULL, NULL, "
            + "NULL, NULL, NULL, NULL, NULL, "
            + "NULL, NULL, NULL)";

        stmt = con.createStatement(); 
        rows = stmt.executeUpdate(sql); 
        con.commit(); 
        stmt.close();

        System.out.println(rows + " row(s) added");

    }

    /** Update a new row */ 
    static void update(boolean commit) throws SQLException {

        Statement stmt; // SQL statement object 
        String sql;     // SQL update command 
        int rows;       // Number of rows inserted 
        String title;   // New title for the update

        if (commit)
           title = "Goof Off"; 
        else
           title = "President";

        sql = "UPDATE Employees " 
            + "SET Title='" + title + "' " 
            + "WHERE EmployeeId = 100";

        stmt = con.createStatement();
        rows = stmt.executeUpdate(sql);

        if (commit)
           con.commit(); 
        else 
           con.rollback();

        stmt.close();
        System.out.println(rows + " row(s) updated");

    }

    /** Delete a row */ 
    static void delete() throws SQLException {

        Statement stmt; // SQL statement object 
        String sql;     // SQL delete command 
        int rows;       // Number of rows inserted

        sql = "DELETE FROM Employees " 
            + "WHERE EmployeeId = 100";

        stmt = con.createStatement(); 
        rows = stmt.executeUpdate(sql); 
        con.commit(); 
        stmt.close();

        System.out.println(rows + " row(s) deleted");

    }

}
